var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var F10120;
(function (F10120) {
    var InterfaceRules = (function (_super) {
        __extends(InterfaceRules, _super);
        function InterfaceRules() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this._canAddPurchaseObject = true;
            return _this;
        }
        InterfaceRules.prototype.SheetP1FormatEventHandler = function (sheetId, row, column, groupIndex) {
            var lastRow = this.Document.Sheets.getValue(sheetId).RowCount - 1;
            if (column < 2) {
                return SheetFormatCollection.Default;
            }
            else if (this.Document.CommonRules.IsFooterRow(sheetId, row)) {
                return SheetFormatCollection.Calc;
            }
            else {
                return SheetFormatCollection.Related;
            }
        };
        InterfaceRules.prototype.SheetNpaFormatEventHandler = function (sheetId, row, column, groupIndex) {
            switch (column) {
                case 0:
                    return SheetFormatCollection.Related;
                case 1:
                case 5:
                    return SheetFormatCollection.Spr;
                default:
                    return SheetFormatCollection.Free;
            }
        };
        InterfaceRules.prototype.FilterKosgu = function () {
            var kosguCode = ObasTableCollection.KosguSprTable.Code.Value || "";
            return (kosguCode !== "212") && this.Document.CommonRules.FilterKosgu();
        };
        InterfaceRules.prototype.AddKosgu212 = function () {
            var formTotalTable = this._document.TableP1;
            var formAnalyticsTable = this._document.AnalyticsDataTable;
            var keyKosgu212 = ObasTableCollection.KosguSprTable.LookupKeyByCode("212");
            if (!formAnalyticsTable.Locate([formAnalyticsTable.Kosgu.ForeignKey.Id], [keyKosgu212])) {
                formAnalyticsTable.AddRow();
                formAnalyticsTable.PostRow();
                formAnalyticsTable.Kosgu.ForeignKey.Value = keyKosgu212;
                if (formTotalTable.IsTotalKeyField.Locate(1)) {
                    for (var i = 1; i <= ObasStageSettings.YearsCount; i++) {
                        var yearFieldId = BaseObasTableFields.YearDataField.GenerateId(i);
                        var otherFieldId = F10120.ObasTableFields.OtherRubField.GenerateId(i);
                        formAnalyticsTable.SetFieldValue(yearFieldId, formTotalTable.GetFieldValue(otherFieldId));
                    }
                }
                else {
                    this.DeleteKosgu212();
                }
            }
        };
        InterfaceRules.prototype.DeleteKosgu212 = function () {
            var formAnalyticsTable = this._document.AnalyticsDataTable;
            var keyKosgu212 = ObasTableCollection.KosguSprTable.LookupKeyByCode("212");
            if (formAnalyticsTable.Locate([formAnalyticsTable.Kosgu.ForeignKey.Id], [keyKosgu212])) {
                formAnalyticsTable.DeleteRow();
            }
        };
        InterfaceRules.prototype.BeforeDocumentSave = function () {
            this.AddKosgu212();
            this.Document.AnalyticSheet.Refresh();
            var result = this.Document.CommonRules.CheckChangeType();
            this.DeleteKosgu212();
            this.Document.AnalyticSheet.Refresh();
            return result;
        };
        InterfaceRules.prototype.SaveKosguTable = function (tableId) {
            this.AddKosgu212();
            this.Document.CommonRules.UpdateMainKosguValues();
            this.DeleteKosgu212();
        };
        InterfaceRules.prototype.DocumentSaveEventHandler = function () {
            var tableP1 = this.Document.TableP1;
            var yearsCount = this.Document.Settings.YearsCount;
            if (tableP1.IsTotalKeyField.Locate(1)) {
                var kosguSprTable = ObasTableCollection.KosguSprTable;
                var data = [];
                var record_1 = {
                    Key: kosguSprTable.LookupKeyByCode("212"),
                    Data: []
                };
                var otherField_1 = F10120.ObasTableFields.OtherRubField;
                var yearField_1 = BaseObasTableFields.YearDataField;
                this.Document.IterateByYears(function (i) {
                    record_1.Data[i] = tableP1.GetFieldValue(otherField_1.GenerateId(i + 1));
                }, false);
                data.push(record_1);
                record_1 = {
                    Key: kosguSprTable.LookupKeyByCode("999"),
                    Data: []
                };
                this.Document.IterateByYears(function (i) {
                    record_1.Data[i] = tableP1.GetFieldValue(yearField_1.GenerateId(i + 1)) -
                        tableP1.GetFieldValue(otherField_1.GenerateId(i + 1));
                }, false);
                data.push(record_1);
                this.Document.CommonRules.UpdateKosguTable(data);
            }
            this.Document.CommonRules.DocumentSaveEventHandler();
        };
        InterfaceRules.prototype.ObjectDescSheetFormatEventHandler = function (sheetId, row, column, groupIndex) {
            switch (column) {
                case 0:
                case 1:
                    return SheetFormatCollection.Related;
                case 2:
                    return SheetFormatCollection.Default;
                case 3:
                case 7:
                    return SheetFormatCollection.Spr;
                default:
                    return SheetFormatCollection.Free;
            }
        };
        InterfaceRules.prototype.ObjectDescSheetCalcEventHandler = function (sheetId, row, column, fieldId) {
            if (BaseObasTableFields.StrCodeField.Equal(fieldId)) {
                return ObasHelper.FillWithCharacter(row + 1, 6);
            }
            return undefined;
        };
        InterfaceRules.prototype.ObjectDescSheetCanAddEventHandler = function (sheetId, row, column, fieldId, rowLevel) {
            return column > 2;
        };
        InterfaceRules.prototype.ObjectDescSheetCanEditEventHandler = function (sheetId, row, column, fieldId, rowLevel) {
            var table = this._document.Sheets.getValue(sheetId).Table;
            if (column > 2) {
                return table.GetKeyBySourceTable(this._document.P5RefObjDescNpaTable) != null;
            }
            else {
                return false;
            }
        };
        InterfaceRules.prototype.ObjectDescSheetCanDeleteEventHandler = function (sheetId, row, column, fieldId, rowLevel) {
            return this.ObjectDescSheetCanEditEventHandler(sheetId, row, column, fieldId, rowLevel);
        };
        InterfaceRules.prototype.FilterOKPD = function () {
            var okpdCode = ObasTableCollection.OkpdSprTable.Code.Value || "";
            return okpdCode.length >= 5;
        };
        InterfaceRules.prototype.FilterNPAStatuses = function () {
            var npaStatusesTable = ObasTableCollection.NpaStatusesSprTable;
            return npaStatusesTable.RecordKey.Value === 1 || npaStatusesTable.RecordKey.Value === 2;
        };
        InterfaceRules.prototype.PurchaseObjectsEditorEditFieldEventHandler = function (sheetId, fieldId, newValue, oldValue) {
            var sourceTable = this._document.PurchaseObjectsSprTable;
            switch (fieldId) {
                case sourceTable.Name.Id:
                case sourceTable.NpaGroupKey.Id:
                    var result = 0;
                    var nameObj = sourceTable.Name.Value;
                    var npaGroupKey = sourceTable.NpaGroupKey.Value;
                    while (sourceTable.Locate([sourceTable.Name.Id, sourceTable.NpaGroupKey.Id], [nameObj, npaGroupKey], true)) {
                        result = result + 1;
                    }
                    sourceTable.ClearLocateFlag();
                    this._canAddPurchaseObject = (result === 1);
                    break;
            }
            return true;
        };
        InterfaceRules.prototype.PurchaseObjectsEditorOnCanCloseEventHandler = function (buttonResult) {
            var sourceTable = this._document.PurchaseObjectsSprTable;
            if (buttonResult === MessageResults.Cancel) {
                return true;
            }
            else {
                if (!this._canAddPurchaseObject) {
                    Client.ShowMessage("\u041E\u0448\u0438\u0431\u043A\u0430", "\u0423\u0436\u0435 \u0441\u0443\u0449\u0435\u0441\u0442\u0432\u0443\u0435\u0442 \u043E\u0431\u044A\u0435\u043A\u0442 \u0437\u0430\u043A\u0443\u043F\u043A\u0438 \u0441 \u0442\u0430\u043A\u0438\u043C \u043D\u0430\u0438\u043C\u0435\u043D\u043E\u0432\u0430\u043D\u0438\u0435\u043C \u0438 \u043F\u043E\u0434\u0433\u0440\u0443\u043F\u043F\u043E\u0439 \u043D\u043E\u0440\u043C\u0430\u0442\u0438\u0432\u043D\u044B\u0445 \u0437\u0430\u0442\u0440\u0430\u0442", MessageIcons.Error);
                }
                return (this._canAddPurchaseObject && buttonResult === MessageResults.Ok);
            }
        };
        return InterfaceRules;
    }(BaseInterfaceRules));
    F10120.InterfaceRules = InterfaceRules;
    var BaseSheet = (function (_super) {
        __extends(BaseSheet, _super);
        function BaseSheet(id, _document) {
            var _this = _super.call(this, id, false) || this;
            _this._document = _document;
            return _this;
        }
        Object.defineProperty(BaseSheet.prototype, "Document", {
            get: function () {
                return this._document;
            },
            enumerable: true,
            configurable: true
        });
        BaseSheet.prototype.GetStrCode = function (row) {
            return ObasHelper.FillWithCharacter(row + 1, BaseSheet._rowCodeLength);
        };
        BaseSheet.prototype.CalcEventHandler = function (sheetId, row, column, fieldId) {
            if (column === BaseSheet._rowCodeColumn) {
                return this.GetStrCode(row);
            }
            return undefined;
        };
        return BaseSheet;
    }(Sheet));
    BaseSheet._rowCodeLength = 3;
    BaseSheet._rowCodeColumn = 1;
    F10120.BaseSheet = BaseSheet;
    var BaseCalcSheet = (function (_super) {
        __extends(BaseCalcSheet, _super);
        function BaseCalcSheet() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BaseCalcSheet.prototype.IsFooterRow = function (row) {
            return this.Document.CommonRules.IsFooterRow(this, row);
        };
        BaseCalcSheet.prototype.FormatEventHandler = function (sheetId, row, column, groupIndex) {
            if (this.IsFooterRow(row)) {
                if (column > BaseCalcSheet._rowCodeColumn) {
                    return SheetFormatCollection.Calc;
                }
                else {
                    return SheetFormatCollection.Default;
                }
            }
            else {
                switch (column) {
                    case 0:
                        return SheetFormatCollection.Free;
                    case 1:
                        return SheetFormatCollection.Default;
                    default:
                        return this.FormatDataColumns(row, column);
                }
            }
        };
        BaseCalcSheet.prototype.CalcEventHandler = function (sheetId, row, column, fieldId) {
            var result = _super.prototype.CalcEventHandler.call(this, sheetId, row, column, fieldId);
            if (result == null) {
                if (this.IsFooterRow(row)) {
                    if (column === 0) {
                        return BaseCalcSheet._totalRowName;
                    }
                    else if (column > BaseCalcSheet._rowCodeColumn) {
                        return this.Document.CommonRules.GetFooterSum(this, row, column);
                    }
                }
            }
            return result;
        };
        return BaseCalcSheet;
    }(BaseSheet));
    BaseCalcSheet._totalRowName = "Всего";
    F10120.BaseCalcSheet = BaseCalcSheet;
    var P2Sheet = (function (_super) {
        __extends(P2Sheet, _super);
        function P2Sheet() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        P2Sheet.prototype.FormatDataColumns = function (row, column) {
            return SheetFormatCollection.Free;
        };
        return P2Sheet;
    }(BaseCalcSheet));
    F10120.P2Sheet = P2Sheet;
    var P3Sheet = (function (_super) {
        __extends(P3Sheet, _super);
        function P3Sheet() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        P3Sheet.prototype.GetColumnBlock = function (column) {
            return ((column - (P3Sheet._rowCodeColumn + 1)) / this.Document.Settings.YearsCount) | 0;
        };
        P3Sheet.prototype.FormatDataColumns = function (row, column) {
            if ((this.GetColumnBlock(column) % 2) === 0) {
                return SheetFormatCollection.Free;
            }
            else {
                return SheetFormatCollection.Calc;
            }
        };
        return P3Sheet;
    }(BaseCalcSheet));
    F10120.P3Sheet = P3Sheet;
    var AnalyticsSheet = (function (_super) {
        __extends(AnalyticsSheet, _super);
        function AnalyticsSheet(id, _document) {
            var _this = _super.call(this, id, false) || this;
            _this._document = _document;
            return _this;
        }
        AnalyticsSheet.prototype.IsTotal = function (row) {
            return (row === 0);
        };
        AnalyticsSheet.prototype.IsRowKosgu212 = function (row) {
            return (row === 1) && this._document.AnalyticsDataTable.IsExistsKosgu212();
        };
        AnalyticsSheet.prototype.FormatEventHandler = function (sheetId, row, column, groupIndex) {
            if (column < AnalyticsSheet._strCodeColumn) {
                if (this.IsTotal(row) || this.IsRowKosgu212(row)) {
                    return SheetFormatCollection.Default;
                }
                else {
                    return SheetFormatCollection.Spr;
                }
            }
            else if (column > AnalyticsSheet._strCodeColumn) {
                if (this.IsTotal(row)) {
                    return SheetFormatCollection.Calc;
                }
                else if (this.IsRowKosgu212(row)) {
                    return SheetFormatCollection.Related;
                }
                else {
                    return SheetFormatCollection.Free;
                }
            }
            else {
                return SheetFormatCollection.Default;
            }
        };
        AnalyticsSheet.prototype.CanEditCellEventHandler = function (sheetId, row, column, fieldId, rowLevel) {
            return ((column > AnalyticsSheet._strCodeColumn) && !this.IsTotal(row) && !this.IsRowKosgu212(row));
        };
        AnalyticsSheet.prototype.CantEditEventHandler = function (sheetId, row, column, fieldId, rowLevel) {
            return (row > 0) && !this.IsRowKosgu212(row);
        };
        AnalyticsSheet.prototype.AnalyticsDataCanCloseEventHandler = function (buttonResult) {
            var sourceTable = this._document.AnalyticsDataTable;
            if (buttonResult === MessageResults.Cancel) {
                return true;
            }
            else {
                var i = 0;
                while (sourceTable.Locate(sourceTable.Kosgu.ForeignKey.Id, sourceTable.Kosgu.ForeignKey.Value, true)) {
                    i++;
                }
                sourceTable.ClearLocateFlag();
                var existsKosgu = (i > 1);
                if (existsKosgu) {
                    Client.ShowMessage("\u041E\u0448\u0438\u0431\u043A\u0430", "\u0423\u0436\u0435 \u0441\u0443\u0449\u0435\u0441\u0442\u0432\u0443\u0435\u0442 \u0441\u0442\u0440\u043E\u043A\u0430 \u0441 \u041A\u041E\u0421\u0413\u0423 = \"" + sourceTable.Kosgu.Code + "\"", MessageIcons.Error);
                }
                else {
                    var needPrefix = (this.RowCount === 1);
                    sourceTable.FillAnalyticsData(needPrefix);
                }
                return (!existsKosgu && buttonResult === MessageResults.Ok);
            }
        };
        AnalyticsSheet._strCodeColumn = 1;
        return AnalyticsSheet;
    }(Sheet));
    F10120.AnalyticsSheet = AnalyticsSheet;
})(F10120 || (F10120 = {}));
